//============================================================================
// Eli_CheckPoint.js
//============================================================================

/*:
@plugindesc v1.2 - Check point system(autosave/load).
@author Eliaquim or Hakuen Studio

@help
Made and tested with Rpg Maker Mv 1.6.2.
Always have a backup from your project!
==============================================================================
Requirements / Dependencies
==============================================================================

    You need Eli’s_Book.js to be somewhere above this plugin.

==============================================================================
INTRODUCTION
==============================================================================

	An autosave or checkpoint system is something that many games today
use. This plugin makes this type of system possible in Rpg Maker Mv with some
extra things.

==============================================================================
FEATURES
==============================================================================

    • Autosave / autoload
    • Autoload option after game over
    • Run a common event after autoload
    • It is not possible to save manually in the auto slot

==============================================================================
HOW TO USE
==============================================================================

Plugin commands

    • AutoSave > autosave the game in the autoslot.
    • AutoLoad > autoload the autoslot.

    Script calls:
    • $gameEli.autoSave()
    • $gameEli.autoLoad()

==============================================================================
CONTACT
==============================================================================

RM Web - https://forums.rpgmakerweb.com/index.php?members/eliaquim.123037/
Centro Rpg Maker - https://centrorpg.com/index.php?action=profile
Instagram - https://www.instagram.com/hakuenstudio
Twitter - https://twitter.com/hakuen_studio
Facebook - https://www.facebook.com/hakuenstudio

==============================================================================
TERMS OF USE
==============================================================================

1. Give the credits to Eliaquim or Hakuen Studio in the credits section of
your game.
2. It can be used in free and commercial games.
3. Do not sell or say that you made this plugin or part of it.
4. Do not redistribute this plugin. Instead, give this link to the download:
https://rakuenzero.itch.io/eli-checkpoint-rpg-maker-mv

==============================================================================
UPDATE LOG
==============================================================================
• Version 1.2 - 18/05/2020
- Add dependency with Eli's_Book.js
- Fixed a bug that prevents the autoload common event from play when loaded
the autoslot via Scene_Load(Continue from title screen).
- Code clean up!

• Version 1.1 - 10/01/2020
- Added autosave/load system!
- Added option to change the help text when the autosave/load slot is
selected.
- Modified plugin name.
- Added option to run a common event when loading the game through the slot
autosave.

• Version 1.0 - 01/09/2020
- Plugin released!

@param autoSlotName
@text AutoSave slot name
@type text
@desc Choose a name for the auto save slot
@default AutoSave

@param autoSaveHelp
@text Scene_Save - Help Text
@type text
@desc Choose a description for the autosave slot in Scene_Save.
@default You can't overwrite an autosave file.

@param autoLoadHelp
@text Scene_Load - Help Text
@type text
@desc Choose a description for the autosave slot in Scene_Load.
@default Continue from your autosave.

@param autoCommonEvent
@text Auto Load Common Event
@type common_event
@desc Choose a common event to play when the autosave file gets loaded.
@default 0

@param autoLoadInGameOver
@text AutoLoad in GameOver
@type boolean
@desc Choose if you want to load the game when the game is over.
@default true

*/

/*:pt
@plugindesc v1.2 - Sistema de check point(autosave/load)
@author Eliaquim or Hakuen Studio

@help
Feito e testado no Rpg Maker 1.6.2
Sempre faça um backup do seu projeto!
==============================================================================
Requerimentos / Dependências
==============================================================================

    Você precisa que Eli’s_Book.js esteja em algum lugar acima desse plugin.
     
==============================================================================
INTRODUÇÃO
==============================================================================

	Um sistema de autosave ou checkpoint é algo que muitos jogos hoje em
dia usam. Esse plugin torna possível esse tipo de sistema no Rpg Maker Mv
junto com algumas funcionalidades extras.

==============================================================================
FUNCIONALIDADES
==============================================================================

Sistema:
    • Autosave/autoload
    • Opção de autoload após o game over
    • Executar um evento comum após o autoload
    • Não é possível salvar manualmente no autoslot

==============================================================================
COMO USAR
==============================================================================

Comandos de plugin:

    • AutoSave > Executa o autosave.
    • AutoLoad > Executa o autoload.

	Script calls:
    • $gameEli.autoSave()
    • $gameEli.autoLoad()

==============================================================================
CONTATO
==============================================================================

RM Web - https://forums.rpgmakerweb.com/index.php?members/eliaquim.123037/
Centro Rpg Maker - https://centrorpg.com/index.php?action=profile
Instagram - https://www.instagram.com/hakuenstudio
Twitter - https://twitter.com/hakuen_studio
Facebook - https://www.facebook.com/hakuenstudio

==============================================================================
TERMOS DE USO
==============================================================================

1. Dê os créditos para Eliaquim ou Hakuen Studio na seção de créditos do seu
jogo.
2. Pode ser usado em jogos gratuitos e comerciais.
3. Não venda e nem diga que foi você que fez esse plugin ou parte dele.
4. Não redistribua esse plugin. Ao invés disso, dê este link para o download:
https://rakuenzero.itch.io/eli-checkpoint-rpg-maker-mv

==============================================================================
ATUALIZAÇÕES
==============================================================================
• Versão 1.2 – 18/05/2020
- Adicionado dependência com Eli's Book.js
- Consertado um bug quando o autoslot era carregado através da Scene_Load não
executava o evento comum de autoload.
- Limpeza no código!
• Versão 1.1 - 10/01/2020
- Adicionado sistema de autosave/load!
- Adicionado opção de mudar o texto de ajuda quando o autosave/load slot é
selecionado.
- Nome do plugin modificado.
- Adicionado opção de rodar um evento comum ao carregar o jogo pelo slot de
autosave.

• Versão 1.0 - 09/01/2020
- Plugin lançado!

@param autoSlotName
@text Nome do slot
@type text
@desc Escolha um nome para ao slot de autoSave/Load
@default AutoSave

@param autoSaveHelp
@text Scene_Save - Texto de ajuda
@type text
@desc Escolha um texto para ser exibido na janela de ajuda quando estiver no autoSlot.
@default Você não pode salvar em cima de um auto slot.

@param autoLoadHelp
@text Scene_Save - Texto de ajuda
@type text
@desc Escolha um texto para ser exibido na janela de ajuda quando estiver no autoSlot.
@default Continue seu jogo pelo auto slot.

@param autoCommonEvent
@text Load common event
@type common_event
@desc Escolha um evento comum para ser executado quando der autoload.
@default 0

@param autoLoadInGameOver
@text AutoLoad no GameOver
@type boolean
@desc Escolha se deseja carregar o jogo no auto slot após o game over.
@default true

*/

// Possibilidade de adicionar todos os tipos de animação como padrão?

"use strict";

var Imported = Imported || {}; 
Imported.Eli_CheckPoint = true;

var Eli = Eli || {}; 
Eli.CheckPoint = Eli.CheckPoint || {}; 

Eli.needBook = function() {
    if(!Eli.alert){
        window.alert(`Eli's_Book.js was not found. 
Please download the latest version for free.`);
        if(confirm) {
            window.open('https://hakuenstudio.itch.io/elis-book-rpg-maker-mv');
        }
        Eli.alert = true;
    }
};

if(!Imported.Eli_Book) {
    Eli.needBook();
}

Eli.CheckPoint.Parameters = PluginManager.parameters('Eli_CheckPoint');
Eli.CheckPoint.Param = Eli.Book.convertParameters(Eli.CheckPoint.Parameters) || {};
delete Eli.CheckPoint.Parameters;

/*==============================================================\\
                            MY FUNCTIONS                          
//==============================================================*/

Eli.CheckPoint.autoLoadCommonEvent = function() {
    if(DataManager.lastAccessedSavefileId() === 1){
        $gameTemp.reserveCommonEvent(Eli.CheckPoint.Param.autoCommonEvent);
    }
};

Eli.CheckPoint.autoLoad = function() {
    if(DataManager.loadGame(1)) {
        SoundManager.playLoad();
        SceneManager._scene.fadeOutAll();
        $gamePlayer.reserveTransfer($gameMap.mapId(), $gamePlayer.x, $gamePlayer.y);
        $gamePlayer.requestMapReload();
        SceneManager.goto(Scene_Map);
        $gameSystem.onAfterLoad();
        Eli.CheckPoint.autoLoadCommonEvent();
    }
};

Eli.CheckPoint.autoSave = function() {
    $gameSystem.onBeforeSave();
    if(DataManager.saveGame(1)) {
        StorageManager.cleanBackup(1);
    }
};

/*==============================================================\\
                            SCENES                          
//==============================================================*/

Eli.CheckPoint.Scene_File_createListWindow = Scene_File.prototype.createListWindow;
Scene_File.prototype.createListWindow = function() {
    Eli.CheckPoint.Scene_File_createListWindow.call(this);
    this._listWindow.setHelpWindow(this._helpWindow);
};

Eli.CheckPoint.Scene_Gameover_gotoTitle = Scene_Gameover.prototype.gotoTitle;
Scene_Gameover.prototype.gotoTitle = function() {
    if(Eli.CheckPoint.Param.autoLoadInGameOver) {
        Eli.CheckPoint.autoLoad();
    }else{
        Eli.CheckPoint.Scene_Gameover_gotoTitle.call(this);
    }
};

Eli.CheckPoint.Scene_Load_terminate = Scene_Load.prototype.terminate;
Scene_Load.prototype.terminate = function() {
    Eli.CheckPoint.Scene_Load_terminate.call(this);
    this.autoLoadSlot();
};

Scene_Load.prototype.autoLoadSlot = function(){
    if(!this.savefileId() === 1) return;
    Eli.CheckPoint.autoLoadCommonEvent();
};

/*==============================================================\\
                            WINDOWS                          
//==============================================================*/

Eli.CheckPoint.Window_SavefileList_drawFileId = Window_SavefileList.prototype.drawFileId;
Window_SavefileList.prototype.drawFileId = function(id, x, y) {
    if (id === 1){
        this.drawText(Eli.CheckPoint.Param.autoSlotName, x, y, 180);
    } else {
        Eli.CheckPoint.Window_SavefileList_drawFileId.call(this, (id), x, y)
    }
};

Eli.CheckPoint.Window_SavefileList_drawItem = Window_SavefileList.prototype.drawItem;
Window_SavefileList.prototype.drawItem = function(index) {
    Eli.CheckPoint.Window_SavefileList_drawItem.call(this, index);
    if(DataManager.isThisGameFile(1) && SceneManager._scene instanceof Scene_Save){
        const id = index + 1;
        const valid = DataManager.isThisGameFile(id-1)
        const info = DataManager.loadSavefileInfo(id);
        const rect = this.itemRectForText(index);
        if (info) {
            this.changePaintOpacity(valid);
            this.drawContents(info, rect, valid);
            this.changePaintOpacity(true);
        }
    }
};

Window_SavefileList.prototype.isCurrentItemEnabled = function() {
    if(SceneManager._scene instanceof Scene_Save && SceneManager._scene._listWindow._index === 0){
        return false;
    }
        return Window_Selectable.prototype.isCurrentItemEnabled.call(this);  
};

Window_SavefileList.prototype.updateHelp = function(){
    const thisHelp = SceneManager._scene._helpWindow;
    const thisList = SceneManager._scene._listWindow._index;
    if(SceneManager._scene instanceof Scene_Save){
        if(thisList === 0){
            thisHelp.setText(Eli.CheckPoint.Param.autoSaveHelp);
        } else {
            thisHelp.setText(TextManager.saveMessage);
        }
    }
    if(SceneManager._scene instanceof Scene_Load){
        if(thisList === 0){
            thisHelp.setText(Eli.CheckPoint.Param.autoLoadHelp);
        } else{
            thisHelp.setText(TextManager.loadMessage);
        }
    }
};

/*==============================================================\\
                        SCRIPT CALLS                          
//==============================================================*/

Game_Eli.prototype.autoLoad = Eli.CheckPoint.autoLoad;
Game_Eli.prototype.autoSave = Eli.CheckPoint.autoSave;

/*==============================================================\\
                        PLUGIN COMMANDS                          
//==============================================================*/

Eli.CheckPoint.Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand;
Game_Interpreter.prototype.pluginCommand = function (command, args) {
    Eli.CheckPoint.Game_Interpreter_pluginCommand.call(this, command, args);
    this.checkPoint_executeCommand(command, args);
};

Game_Interpreter.prototype.checkPoint_executeCommand = function(command, args){
    const allCommands = {
        AUTOSAVE: 'checkPoint_AutoSave',
        AUTOLOAD: 'checkPoint_AutoLoad',
    };
    const result = allCommands[command.toUpperCase()];
    if(result) this[result](args);
};

Game_Interpreter.prototype.checkPoint_AutoSave = function(args){
    Eli.CheckPoint.autoSave();
};

Game_Interpreter.prototype.checkPoint_AutoLoad = function(args){
    Eli.CheckPoint.autoLoad();
};