//==========================================================================
// Eli_PlayTime.js
//==========================================================================

/*:
@plugindesc v1.2 - Pause the PlayTime.
@author Eliaquim or Rakuen Zero

@help

==============================================================================
Introduction
==============================================================================
Sometimes you want the game to not count the PlayTime. For example, if you 
made a title screen in a map(with events) the game will already start counting 
the playtime.
With this plugin you can Pause game time(PlayTime, not the Timer) counting by 
turning on a switch or using a plugin command.

============================================================================
How to use
============================================================================
You can choose to start the game not counting the playtime in the plugin
parameters. You can change that in mid-game using plugin commands:
resume_playtime
pause_playtime

Or you can use a switch to pause or resume playtime.

Keep in mind that the plugin command has a higher priority than the switch.

==============================================================================
Contact
==============================================================================
RM Web - https://forums.rpgmakerweb.com/index.php?members/eliaquim.123037/
Centro Rpg Maker - https://centrorpg.com/index.php?action=profile
Instagram - https://www.instagram.com/rakuen.zero
Twitter - https://twitter.com/rakuenzero
Facebook - https://www.facebook.com/rakuenzero

============================================================================
Terms of Use
============================================================================
 
1. Give the credits to Eliaquim or Rakuen Zero in the credits section of 
your game.
2. It can be used in free and commercial games.
3. Do not sell or say that you made this plugin.
4. Do not redistribute this plugin. Instead, give this link to the download:
https://rakuenzero.itch.io/eli-playtime

============================================================================
Update Log
============================================================================
Version 1.2 - 01/01/2020
- Add plugin command to reset playtime.
- bug fix.

Version 1.1 - 12/30/2019
- Add compatibility with community code 1.3b

Version 1.0 - 10/29/2019
- Released!


@param Switch to pause time
@type switch
@desc If this swith is on, the PlayTime will be paused.
@default

@param Start game without counting the PlayTime
@type boolean
@on Yes
@off No
@desc Choose to start the game with time paused or not.
@default false

*/

/*:pt
@plugindesc v1.2 - Pause a contagem de tempo do jogo.
@author Eliaquim or Rakuen Zero

@help

==============================================================================
Introdução
==============================================================================
Por algum motivo você pode querer pausar o tempo de jogo. Por exemplo, se 
fizer uma tela de título por eventos(em um mapa), poderá querer que o tempo 
não conte neste mapa.
Com esse plugin você pode fazer isso usando um comando de plugin ou uma switch.

OBS: Este plugin se refere ao PlayTime não ao cronômetro.

============================================================================
Como usar
============================================================================
Você pode escolher por começar o jogo já com o tempo pausado através dos
parâmetros de plugin. Você poderá escolher dentro do jogo quando quer que a
contagem comece ou pause e até resete através dos comandos de plugin:
resume_playtime
pause_playtime
reset_playtime

Ou pode usar uma switch para pausar ou resumir a contagem.

Tenha em mente que o comando de plugin tem prioridade sobre a switch.

==============================================================================
Contato
==============================================================================
RM Web - https://forums.rpgmakerweb.com/index.php?members/eliaquim.123037/
Centro Rpg Maker - https://centrorpg.com/index.php?action=profile
Instagram - https://www.instagram.com/rakuen.zero
Twitter - https://twitter.com/rakuenzero
Facebook - https://www.facebook.com/rakuenzero

============================================================================
Termos de uso
============================================================================
 
1. Dê os créditos para Eliaquim ou Rakuen Zero na seção de créditos do 
seu jogo.
2. Pode ser usado em jogos gratuitos e comerciais.
3. Não venda e nem diga que foi você que fez esse plugin.
4. Não redistribua esse plugin. Ao invés disso, dê este link para o download:
https://rakuenzero.itch.io/eli-playtime

============================================================================
Log de atualizações
============================================================================
Versão 1.2 - 01/01/2020
- Adicionado comando de plugin para resetar o tempo de jogo.
- Conserto de bugs.

Versão 1.1 - 30/12/2019
- Adicionado compatibilidade com a versão do rpg_core community 1.3b

Versão 1.0 - 29/10/2019
- Plugin lançado!


@param Switch to pause time
@text Switch para pausar o tempo
@type switch
@desc Se essa switch estiver ON, o tempo ficará pausado.
@default 0

@param Start game without counting the PlayTime
@text Começar o jogo sem a contagem de tempo?
@type boolean
@on Sim
@off Não
@desc Escolha começar o jogo com a contagem pausada ou não.
@default false

*/

"use strict";
var Imported = Imported || {}; 
Imported.Eli_PlayTime = true;

var Eli = Eli || {}; 
Eli.PlayTime = Eli.PlayTime || {}; 

Eli.Parameters = PluginManager.parameters('Eli_PlayTime');
Eli.Param = Eli.Param || {};

Eli.Param.PlayTime = { 
    PauseTimeSw: Number(Eli.Parameters['Switch to pause time']),
    PauseTime: JSON.parse(Eli.Parameters['Start game without counting the PlayTime'])
};

// Overwrite default function
Eli.PlayTime.Graphics_render = Graphics.render;
Graphics.render = function(stage) {
    if($gameSwitches.value(Eli.Param.PlayTime.PauseTimeSw) || Eli.Param.PlayTime.PauseTime) { 
        if (this._skipCount <= 0 ) {
            const startTime = Date.now();
            if (stage) {
                this._renderer.render(stage);
                if (this._renderer.gl && this._renderer.gl.flush) {
                    this._renderer.gl.flush();
                }
            }
            const endTime = Date.now();
            const elapsed = endTime - startTime;
            this._skipCount = Math.min(Math.floor(elapsed / 15), this._maxSkip);
            this._rendered = true;
        } else {
            this._skipCount--;
            this._rendered = false;
        }
    } else {
            Eli.PlayTime.Graphics_render.call(this, stage);
    }
};

SceneManager.updateFrameCount = function() {
	if($gameSwitches.value(Eli.Param.PlayTime.PauseTimeSw) || Eli.Param.PlayTime.PauseTime) { 
       //do nothing
	} else {
		this._frameCount++;
	}
};

Eli.PlayTime.Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand;
Game_Interpreter.prototype.pluginCommand = function (command, args) {
Eli.PlayTime.Game_Interpreter_pluginCommand.call(this, command, args);
let commandLower = command.toLowerCase();
    if (commandLower === "pause_playtime") {
		return Eli.Param.PlayTime.PauseTime = true;
    }
    if (commandLower === "resume_playtime") {
		return Eli.Param.PlayTime.PauseTime = false;
	}
	if (commandLower === "reset_playtime") {
		SceneManager._frameCount = 0;
		Graphics.frameCount = 0;
	}
};